/*
  Copyright (c) 2016, Antonio SJ Musumeci <trapexit@spawn.link>

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <fuse.h>

#include <string>

#include "config.hpp"
#include "errno.hpp"
#include "fileinfo.hpp"
#include "fs_base_close.hpp"
#include "fs_base_fadvise.hpp"

static
int
_release(FileInfo   *fi,
         const bool  dropcacheonclose)
{
  // according to Feh of nocache calling it once doesn't always work
  // https://github.com/Feh/nocache
  if(dropcacheonclose)
    {
      fs::fadvise_dontneed(fi->fd);
      fs::fadvise_dontneed(fi->fd);
    }

  fs::close(fi->fd);

  delete fi;

  return 0;
}

namespace mergerfs
{
  namespace fuse
  {
    int
    release(const char     *fusepath,
            fuse_file_info *ffi)
    {
      const Config &config = Config::get();
      FileInfo     *fi     = reinterpret_cast<FileInfo*>(ffi->fh);

      return _release(fi,config.dropcacheonclose);
    }
  }
}
